@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #ifndef _ND_PLUGIN_H
 #define _ND_PLUGIN_H
 
-#define _ND_PLUGIN_VER  0x20180813
+#define _ND_PLUGIN_VER  0x20211111
 
 #define ndStartDetectionThreads() kill(getpid(), SIGUSR1)
 #define ndStopDetectionThreads()  kill(getpid(), SIGUSR2)
@@ -27,8 +27,10 @@
     ndPlugin *ndPluginInit(const string &tag) { \
         class_name *p = new class_name(tag); \
         if (p == NULL) return NULL; \
-        if (p->GetType() != ndPlugin::TYPE_TASK && \
-            p->GetType() != ndPlugin::TYPE_SERVICE) { \
+        if (p->GetType() != ndPlugin::TYPE_SINK_TASK && \
+            p->GetType() != ndPlugin::TYPE_SINK_SERVICE && \
+            p->GetType() != ndPlugin::TYPE_DETECTION && \
+            p->GetType() != ndPlugin::TYPE_STATS) { \
                 nd_printf("Invalid plugin type detected during init: %s\n", \
                     tag.c_str()); \
                 delete p; \
@@ -58,15 +60,37 @@ public:
 
     virtual void *Entry(void) = 0;
 
+    virtual void GetVersion(string &version) = 0;
+
+    enum ndPluginEvent
+    {
+        EVENT_RELOAD,
+        EVENT_STATUS_UPDATE
+    };
+
+    virtual void ProcessEvent(ndPluginEvent event, void *param = NULL) { };
+
     enum ndPluginType
     {
         TYPE_BASE,
-        TYPE_SERVICE,
-        TYPE_TASK,
+        TYPE_SINK_SERVICE,
+        TYPE_SINK_TASK,
+        TYPE_DETECTION,
+        TYPE_STATS,
     };
 
     ndPluginType GetType(void) { return type; };
 
+protected:
+    ndPluginType type;
+};
+
+class ndPluginSink : public ndPlugin
+{
+public:
+    ndPluginSink(const string &tag);
+    virtual ~ndPluginSink();
+
     virtual void SetParams(const string uuid_dispatch, const ndJsonPluginParams &params);
 
     virtual void GetReplies(
@@ -88,21 +112,20 @@ protected:
         Unlock();
     }
 
-    ndPluginType type;
     ndPluginFiles files;
     ndPluginFiles data;
     ndPluginParams params;
     ndPluginReplies replies;
 };
 
-class ndPluginService : public ndPlugin
+class ndPluginService : public ndPluginSink
 {
 public:
     ndPluginService(const string &tag);
     virtual ~ndPluginService();
 };
 
-class ndPluginTask : public ndPlugin
+class ndPluginTask : public ndPluginSink
 {
 public:
     ndPluginTask(const string &tag);
@@ -124,6 +147,29 @@ protected:
     string uuid_dispatch;
 };
 
+class ndPluginDetection : public ndPlugin
+{
+public:
+    ndPluginDetection(const string &tag);
+    virtual ~ndPluginDetection();
+
+    virtual void ProcessFlow(ndFlow *flow) = 0;
+
+protected:
+};
+
+class ndPluginStats : public ndPlugin
+{
+public:
+    ndPluginStats(const string &tag);
+    virtual ~ndPluginStats();
+
+    virtual void ProcessStats(const nd_packet_stats &pkt_totals) = 0;
+    virtual void ProcessStats(const string &iface, const nd_packet_stats *pkt_stats, const nd_flow_map *flows) = 0;
+
+protected:
+};
+
 #ifdef _ND_INTERNAL
 
 class ndPluginLoader
