@@ -74,7 +74,6 @@ static u_int32_t ndpi_ssl_refine_master_protocol(struct ndpi_detection_module_st
     if(packet->tcp != NULL) {
         switch(protocol) {
             case NDPI_PROTOCOL_SSL:
-            case NDPI_PROTOCOL_SSL_NO_CERT:
                 {
                     /*
                        In case of SSL there are probably sub-protocols
@@ -286,6 +285,8 @@ int getTLScertificate(struct ndpi_detection_module_struct *ndpi_struct,
                 if(handshake_protocol == 0x02) {
                     u_int16_t offset = base_offset, extension_len, j;
                     u_int8_t  session_id_len = packet->payload[offset];
+                    if (offset < total_len)
+                        session_id_len = packet->payload[offset];
 
 #ifdef DEBUG_TLS
                     printf("SSL Server Hello [version: 0x%04X]\n", tls_version);
@@ -304,6 +305,9 @@ https://networkengineering.stackexchange.com/questions/55752/why-does-wireshark-
                             offset += session_id_len+1;
                     }
 
+                    if((offset+3) > packet->payload_packet_len)
+                        return(0); /* Not found */
+
                     ja3.num_cipher = 1, ja3.cipher[0] = ntohs(*((u_int16_t*)&packet->payload[offset]));
                     flow->protos.stun_ssl.ssl.server_unsafe_cipher = ndpi_is_safe_ssl_cipher(ja3.cipher[0]);
                     flow->protos.stun_ssl.ssl.server_cipher = ja3.cipher[0];
@@ -1091,7 +1095,7 @@ int sslTryAndRetrieveServerCertificate(struct ndpi_detection_module_struct *ndpi
                     && (flow->protos.stun_ssl.ssl.server_certificate[0] != '\0'))
                 /* 0 means we've done processing extra packets (since we found what we wanted) */
                 flow->extra_packets_func = NULL; /* We're good now */
-                return 0;
+            return 0;
 #endif
         }
 
@@ -1101,13 +1105,13 @@ int sslTryAndRetrieveServerCertificate(struct ndpi_detection_module_struct *ndpi
                         && (flow->l4.tcp.seen_syn)
                         && (flow->l4.tcp.seen_syn_ack)
                         && (flow->l4.tcp.seen_ack) /* We have seen the 3-way handshake */
-                    /*  && flow->l4.tcp.tls_srv_cert_fingerprint_processed) */
+                        /*  && flow->l4.tcp.tls_srv_cert_fingerprint_processed) */
                     /* || (flow->protos.stun_ssl.ssl.ja3_server[0] != '\0') */
               )) {
-                /* We're done processing extra packets since we've probably checked all possible cert packets */
-                flow->extra_packets_func = NULL; /* We're good now */
-                return(rc);
-            }
+                  /* We're done processing extra packets since we've probably checked all possible cert packets */
+                  flow->extra_packets_func = NULL; /* We're good now */
+                  return(rc);
+              }
         }
     }
 #endif
@@ -1173,20 +1177,20 @@ int tlsDetectProtocolFromCertificate(struct ndpi_detection_module_struct *ndpi_s
                             || flow->l4.tcp.seen_ack /* We have seen the 3-way handshake */)
 #endif
                         /* && (flow->l4.tcp.tls_srv_cert_fingerprint_processed */
-                        && (flow->l4.tcp.ssl_seen_client_cert
-                            || flow->l4.tcp.ssl_seen_server_cert
-                            || packet->ssl_certificate_detected)
-               )
-                    /*
-                       || ((flow->l4.tcp.ssl_seen_certificate == 1)
-                       && (flow->l4.tcp.ssl_seen_server_cert == 1)
-                       && (flow->protos.stun_ssl.ssl.server_certificate[0] != '\0'))
-                       */
-                    /* || ((flow->l4.tcp.ssl_seen_client_cert == 1) && (flow->protos.stun_ssl.ssl.client_certificate[0] != '\0')) */
-              ) {
-                ndpi_int_ssl_add_connection(ndpi_struct, flow, NDPI_PROTOCOL_SSL);
-            }
-        }
+                            && (flow->l4.tcp.ssl_seen_client_cert
+                                    || flow->l4.tcp.ssl_seen_server_cert
+                                    || packet->ssl_certificate_detected)
+                            )
+                        /*
+                           || ((flow->l4.tcp.ssl_seen_certificate == 1)
+                           && (flow->l4.tcp.ssl_seen_server_cert == 1)
+                           && (flow->protos.stun_ssl.ssl.server_certificate[0] != '\0'))
+                           */
+                        /* || ((flow->l4.tcp.ssl_seen_client_cert == 1) && (flow->protos.stun_ssl.ssl.client_certificate[0] != '\0')) */
+               ) {
+                    ndpi_int_ssl_add_connection(ndpi_struct, flow, NDPI_PROTOCOL_SSL);
+                    }
+                    }
     }
 
     return(0);
@@ -1279,7 +1283,7 @@ no_check_for_ssl_payload:
                 && (!(flow->l4.tcp.ssl_seen_client_cert && flow->l4.tcp.ssl_seen_server_cert))) {
             /* SSL without certificate (Skype, Ultrasurf?) */
             NDPI_LOG_INFO(ndpi_struct, "found ssl NO_CERT\n");
-            ndpi_int_ssl_add_connection(ndpi_struct, flow, NDPI_PROTOCOL_SSL);
+            ndpi_int_ssl_add_connection(ndpi_struct, flow, NDPI_PROTOCOL_SSL_NO_CERT);
         } else if((packet->ssl_certificate_num_checks >= 3)
                 /* && flow->l4.tcp.tls_srv_cert_fingerprint_processed */ ) {
             NDPI_LOG_INFO(ndpi_struct, "found ssl\n");
