@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,9 +29,7 @@
 #include <unordered_map>
 #include <list>
 #include <vector>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
 #include <algorithm>
 
@@ -169,6 +167,9 @@ using namespace std;
 // Enable to log discarded packets
 //#define _ND_LOG_PKT_DISCARD     1
 
+// Enable to log discarded flows
+//#define _ND_LOG_FLOW_DISCARD    1
+
 // Enable DNS response debug logging
 //#define _ND_LOG_DNS_RESPONSE    1
 
@@ -182,6 +183,7 @@ using namespace std;
 #define _ND_DISSECT_GTP       1
 
 extern nd_global_config nd_config;
+extern atomic_uint nd_flow_count;
 
 struct __attribute__((packed)) nd_mpls_header_t
 {
@@ -395,7 +397,7 @@ void *ndCaptureThread::Entry(void)
     struct ifreq ifr;
 
     do {
-        if (pcap == NULL && ! terminate) {
+        if (pcap == NULL && ! ShouldTerminate()) {
             if (nd_ifreq(tag, SIOCGIFFLAGS, &ifr) == -1) {
                 sleep(1);
                 continue;
@@ -436,7 +438,7 @@ void *ndCaptureThread::Entry(void)
             pkt_queue.pop();
         }
 
-        if (! terminate) {
+        if (! ShouldTerminate()) {
             if (pcap_fd != -1) {
                 int rc, max_fd = 0;
                 struct timeval tv;
@@ -520,13 +522,13 @@ void *ndCaptureThread::Entry(void)
                     tag.c_str(), pcap_file.c_str(), pkt_queue.size());
                 pcap_close(pcap);
                 pcap = NULL;
-                terminate = true;
+                Terminate();
                 pcap_fd = -1;
                 break;
             }
         }
     }
-    while (terminate == false || ! pkt_queue.empty());
+    while (ShouldTerminate() == false || ! pkt_queue.empty());
 
     nd_dprintf(
         "%s: capture ended on CPU: %lu\n", tag.c_str(), cpu >= 0 ? cpu : 0);
@@ -645,7 +647,7 @@ void ndCaptureThread::DumpFlows(void)
 
     for (nd_flow_map::const_iterator i = flows->begin(); i != flows->end(); i++) {
 
-        if (i->second->flags.detection_complete == false) continue;
+        if (i->second->flags.detection_complete.load() == false) continue;
         if (! ND_FLOW_DUMP_UNKNOWN &&
             i->second->detected_protocol.master_protocol == NDPI_PROTOCOL_UNKNOWN) continue;
 
@@ -745,7 +747,7 @@ void ndCaptureThread::ProcessPacket(void)
         default:
 #ifdef _ND_LOG_PKT_DISCARD
         stats->pkt.discard++;
-        stats->pkt.discard_bytes += pkt_header->caplen;
+        stats->pkt.discard_bytes += pkt_header->len;
         nd_dprintf("%s: discard: Unsupported BSD loopback encapsulation type: %lu\n",
             tag.c_str(), ntohl(*((uint32_t *)pkt_data)));
 #endif
@@ -767,7 +769,7 @@ void ndCaptureThread::ProcessPacket(void)
             (hdr_eth->ether_dhost[0] == 0x01 && hdr_eth->ether_dhost[1] == 0x80 &&
             hdr_eth->ether_dhost[2] == 0xC2)) {
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->caplen;
+            stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_dprintf("%s: discard: STP protocol.\n", tag.c_str());
 #endif
@@ -791,7 +793,7 @@ void ndCaptureThread::ProcessPacket(void)
 
     default:
         stats->pkt.discard++;
-        stats->pkt.discard_bytes += pkt_header->caplen;
+        stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
         nd_dprintf("%s: discard: Unsupported datalink type: 0x%x\n",
             tag.c_str(), (unsigned)pcap_datalink_type);
@@ -831,7 +833,7 @@ void ndCaptureThread::ProcessPacket(void)
             );
             if (ppp_proto != PPP_IP && ppp_proto != PPP_IPV6) {
                 stats->pkt.discard++;
-                stats->pkt.discard_bytes += pkt_header->caplen;
+                stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
                 nd_dprintf("%s: discard: unsupported PPP protocol: 0x%04hx\n",
                     tag.c_str(), ppp_proto);
@@ -844,7 +846,7 @@ void ndCaptureThread::ProcessPacket(void)
         else if (type == ETHERTYPE_PPPOEDISC) {
             stats->pkt.pppoe++;
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->caplen;
+            stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_dprintf("%s: discard: PPPoE discovery protocol.\n", tag.c_str());
 #endif
@@ -878,7 +880,7 @@ nd_process_ip:
         if (pkt_header->caplen - l2_len < sizeof(struct ip)) {
             // XXX: header too small
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->caplen;
+            stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_dprintf("%s: discard: header too small\n", tag.c_str());
 #endif
@@ -889,7 +891,7 @@ nd_process_ip:
             // XXX: fragmented packets are not supported
             stats->pkt.frags++;
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->caplen;
+            stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_dprintf("%s: discard: fragmented 0x3FFF\n", tag.c_str());
 #endif
@@ -900,7 +902,7 @@ nd_process_ip:
             // XXX: fragmented packets are not supported
             stats->pkt.frags++;
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->caplen;
+            stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_dprintf("%s: discard: fragmented 0x1FFF\n", tag.c_str());
 #endif
@@ -909,24 +911,26 @@ nd_process_ip:
 
         if (l3_len > (pkt_header->caplen - l2_len)) {
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->caplen;
+            stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_dprintf("%s: discard: l3_len[%hu] > (pkt_header->caplen[%hu] - l2_len[%hu])(%hu)\n",
                 tag.c_str(), l3_len, pkt_header->caplen, l2_len, pkt_header->caplen - l2_len);
 #endif
             return;
         }
+#if 0
+        // XXX: Disabled, drops all large packets.  Investigate.
 
         if ((pkt_header->caplen - l2_len) < ntohs(hdr_ip->ip_len)) {
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->caplen;
+            stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_dprintf("%s: discard: (pkt_header->caplen[%hu] - l2_len[%hu](%hu)) < hdr_ip->ip_len[%hu]\n",
                 tag.c_str(), pkt_header->caplen, l2_len, pkt_header->caplen - l2_len, ntohs(hdr_ip->ip_len));
 #endif
             return;
         }
-
+#endif
         addr_cmp = memcmp(&hdr_ip->ip_src, &hdr_ip->ip_dst, 4);
 
         if (addr_cmp < 0) {
@@ -962,7 +966,7 @@ nd_process_ip:
 
         if (ndpi_handle_ipv6_extension_headers(NULL, &l4, &l4_len, &flow.ip_protocol)) {
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->caplen;
+            stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_dprintf("%s: discard: Error walking IPv6 extensions.\n", tag.c_str());
 #endif
@@ -1003,7 +1007,7 @@ nd_process_ip:
     else {
         // XXX: Warning: unsupported IP protocol version (IPv4/6 only)
         stats->pkt.discard++;
-        stats->pkt.discard_bytes += pkt_header->caplen;
+        stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
         nd_dprintf("%s: discard: invalid IP protocol version: %hhx\n",
             tag.c_str(), pkt_data[l2_len]);
@@ -1221,6 +1225,17 @@ nd_process_ip:
         }
     }
     else {
+        if (nd_config.max_flows > 0 && nd_flow_count + 1 > nd_config.max_flows) {
+            stats->pkt.discard++;
+            stats->pkt.discard_bytes += pkt_header->len;
+            stats->flow.dropped++;
+#ifdef _ND_LOG_FLOW_DISCARD
+            nd_dprintf("%s: discard: maximum flows exceeded: %u\n",
+                tag.c_str(), nd_config.max_flows);
+#endif
+            return;
+        }
+
         nf = new ndFlow(flow);
         if (nf == NULL) throw ndCaptureThreadException(strerror(ENOMEM));
 
@@ -1233,6 +1248,8 @@ nd_process_ip:
             throw ndCaptureThreadException(strerror(EINVAL));
         }
 
+        nd_flow_count++;
+
         // New flow inserted, initialize...
         nf->ts_first_seen = ts_pkt;
 
@@ -1305,13 +1322,14 @@ nd_process_ip:
     if (! nf->ts_first_update)
         nf->ts_first_update = ts_pkt;
 
-    if (nf->ip_protocol == IPPROTO_TCP &&
-        (hdr_tcp->th_flags & TH_FIN || hdr_tcp->th_flags & TH_RST)) {
+    if (nf->ip_protocol == IPPROTO_TCP) {
         if (hdr_tcp->th_seq <= nf->tcp_last_seq) {
             stats->pkt.tcp_seq_error++;
             nf->tcp_last_seq = hdr_tcp->th_seq;
         }
-        nf->flags.tcp_fin = true;
+
+        if (hdr_tcp->th_flags & TH_FIN) nf->flags.tcp_fin = true;
+        if (hdr_tcp->th_flags & TH_RST) stats->pkt.tcp_resets++;
     }
 
     if (dhc != NULL && pkt != NULL && pkt_len > sizeof(struct nd_dns_header_t)) {
@@ -1356,17 +1374,31 @@ nd_process_ip:
                     *fi.first->second += *nf;
 
                     nd_dprintf("%s: delete rehashed DNS flow: %lu packets, detection complete: %s\n",
-                        tag.c_str(), nf->total_packets, (nf->flags.detection_complete) ? "yes" : "no");
+                        tag.c_str(), nf->total_packets, (nf->flags.detection_complete.load()) ? "yes" : "no");
                     delete nf;
 
                     return;
                 }
             }
+#else
+            if (is_query && nf->flags.detection_complete.load()) {
+                nf->flags.dhc_hit = 0;
+                nf->flags.detection_complete = 0;
+                // XXX: Moot...
+                //if (fi.first != flows->end()) {
+                    nf->lower_bytes = flow.lower_bytes;
+                    nf->upper_bytes = flow.upper_bytes;
+                    nf->lower_packets = flow.lower_packets;
+                    nf->upper_packets = flow.upper_packets;
+                    nf->total_packets = flow.total_packets;
+                    nf->detection_packets = 0;
+                //}
+            }
 #endif
         }
     }
 
-    if (! nf->flags.detection_complete
+    if (! nf->flags.detection_complete.load()
         || (nf->ip_protocol == IPPROTO_UDP &&
             nf->total_packets <= nd_config.max_udp_pkts)
         || (nf->ip_protocol == IPPROTO_TCP &&
