@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,9 +25,7 @@
 #include <list>
 #include <vector>
 #include <sstream>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
 
 #include <sys/stat.h>
@@ -47,6 +45,10 @@
 #include <linux/netlink.h>
 #endif
 
+#define __FAVOR_BSD 1
+#include <netinet/tcp.h>
+#undef __FAVOR_BSD
+
 #include <pcap/pcap.h>
 
 #include <libmnl/libmnl.h>
@@ -114,8 +116,8 @@ static int nd_ct_netlink_callback(const struct nlmsghdr *nlh, void *data)
     return MNL_CB_OK;
 }
 
-ndConntrackThread::ndConntrackThread()
-    : ndThread("nd-conntrack", -1),
+ndConntrackThread::ndConntrackThread(int16_t cpu)
+    : ndThread("nd-conntrack", (long)cpu),
     ctfd(-1), cth(NULL), cb_registered(-1)
 {
     cth = nfct_open(NFNL_SUBSYS_CTNETLINK, NFCT_ALL_CT_GROUPS);
@@ -148,7 +150,7 @@ ndConntrackThread::ndConntrackThread()
 
     DumpConntrackTable();
 
-    nd_debug_printf("%s: Created.\n", tag.c_str());
+    nd_dprintf("%s: Created.\n", tag.c_str());
 }
 
 ndConntrackThread::~ndConntrackThread()
@@ -164,7 +166,7 @@ ndConntrackThread::~ndConntrackThread()
     for (nd_ct_flow_map::const_iterator i = ct_flow_map.begin();
         i != ct_flow_map.end(); i++) delete i->second;
 
-    nd_debug_printf("%s: Destroyed.\n", tag.c_str());
+    nd_dprintf("%s: Destroyed.\n", tag.c_str());
 }
 
 void ndConntrackThread::DumpConntrackTable(void)
@@ -223,7 +225,7 @@ void ndConntrackThread::DumpConntrackTable(void)
 
     mnl_socket_close(nl);
 
-    nd_debug_printf("%s: Loaded %lu conntrack entries.\n",
+    nd_dprintf("%s: Loaded %lu conntrack entries.\n",
         tag.c_str(), ct_id_map.size());
 }
 
@@ -258,7 +260,7 @@ void *ndConntrackThread::Entry(void)
         }
     }
 
-    nd_debug_printf("%s: Exit.\n", tag.c_str());
+    nd_dprintf("%s: Exit.\n", tag.c_str());
     return NULL;
 }
 
@@ -279,7 +281,7 @@ void ndConntrackThread::ProcessConntrackEvent(
 #ifdef _ND_DEBUG_CONNTRACK
         id_iter = ct_id_map.find(id);
         if (id_iter != ct_id_map.end()) {
-            nd_debug_printf("%s: [N:%u] ID exists for new flow.\n",
+            nd_dprintf("%s: [N:%u] ID exists for new flow.\n",
                 tag.c_str(), id);
         }
 #endif
@@ -297,7 +299,7 @@ void ndConntrackThread::ProcessConntrackEvent(
         flow_iter = ct_flow_map.find(ct_flow->digest);
         if (flow_iter != ct_flow_map.end()) {
 #ifdef _ND_DEBUG_CONNTRACK
-            nd_debug_printf("%s: [N:%u] Digest found in flow map.\n",
+            nd_dprintf("%s: [N:%u] Digest found in flow map.\n",
                 tag.c_str(), id);
 #endif
             delete flow_iter->second;
@@ -314,7 +316,7 @@ void ndConntrackThread::ProcessConntrackEvent(
 
         flow_iter = ct_flow_map.find(id_iter->second);
         if (flow_iter == ct_flow_map.end()) {
-            nd_debug_printf("%s: [U:%u] Digest not found in flow map.\n",
+            nd_dprintf("%s: [U:%u] Digest not found in flow map.\n",
                 tag.c_str(), id);
             ct_id_map.erase(id_iter);
             goto Unlock_ProcessConntrackEvent;
@@ -326,7 +328,7 @@ void ndConntrackThread::ProcessConntrackEvent(
 
         if (ct_flow->digest != id_iter->second) {
 #ifdef _ND_DEBUG_CONNTRACK
-            nd_debug_printf("%s: [U:%u] Flow hash updated.\n",
+            nd_dprintf("%s: [U:%u] Flow hash updated.\n",
                 tag.c_str(), id);
 #endif
             ct_flow_map.erase(flow_iter);
@@ -360,7 +362,7 @@ void ndConntrackThread::ProcessConntrackEvent(
 #if 0
         char buffer[1024];
         nfct_snprintf(buffer, sizeof(buffer), ct, type, NFCT_O_PLAIN, NFCT_OF_TIME);
-        nd_debug_printf("%s: %02x [%u] %s\n", tag.c_str(), type, id, buffer);
+        nd_dprintf("%s: %02x [%u] %s\n", tag.c_str(), type, id, buffer);
 #endif
     }
 #endif
@@ -524,7 +526,7 @@ void ndConntrackThread::PrintFlow(ndFlow *flow, string &text)
     text = os.str();
 }
 
-void ndConntrackThread::ClassifyFlow(ndFlow *flow)
+void ndConntrackThread::UpdateFlow(ndFlow *flow)
 {
     sha1 ctx;
     string digest;
@@ -577,6 +579,11 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
 
         ndConntrackFlow *ct_flow = flow_iter->second;
 
+        flow->ct_id = ct_flow->id;
+        flow->ct_mark = ct_flow->mark;
+
+        ct_flow->updated_at = time(NULL);
+
         switch (ct_flow->l3_proto) {
         case AF_INET:
             sa_orig_src = reinterpret_cast<struct sockaddr_in *>(
@@ -591,18 +598,12 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
             {
                 string flow_text;
                 PrintFlow(ct_flow, flow_text, false, true);
-                nd_debug_printf("%s: %s\n", tag.c_str(), flow_text.c_str());
+                nd_dprintf("%s: %s\n", tag.c_str(), flow_text.c_str());
             }
 #endif
             if (memcmp(sa_orig_src, sa_repl_dst, sizeof(struct sockaddr_in)) ||
-                memcmp(sa_orig_dst, sa_repl_src, sizeof(struct sockaddr_in))) {
+                memcmp(sa_orig_dst, sa_repl_src, sizeof(struct sockaddr_in)))
                 flow->flags.ip_nat = true;
-#ifdef _ND_USE_CONNTRACK
-                flow->ct_id = ct_flow->id;
-                flow->ct_mark = ct_flow->mark;
-#endif
-                ct_flow->updated_at = time(NULL);
-            }
 
             break;
 
@@ -619,18 +620,12 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
             {
                 string flow_text;
                 PrintFlow(ct_flow, flow_text, false, true);
-                nd_debug_printf("%s: %s\n", tag.c_str(), flow_text.c_str());
+                nd_dprintf("%s: %s\n", tag.c_str(), flow_text.c_str());
             }
 #endif
             if (memcmp(sa6_orig_src, sa6_repl_dst, sizeof(struct sockaddr_in6)) ||
-                memcmp(sa6_orig_dst, sa6_repl_src, sizeof(struct sockaddr_in6))) {
+                memcmp(sa6_orig_dst, sa6_repl_src, sizeof(struct sockaddr_in6)))
                 flow->flags.ip_nat = true;
-#ifdef _ND_USE_CONNTRACK
-                flow->ct_id = ct_flow->id;
-                flow->ct_mark = ct_flow->mark;
-#endif
-                ct_flow->updated_at = time(NULL);
-            }
 
             break;
         }
@@ -662,7 +657,7 @@ void ndConntrackThread::DumpStats(void)
 {
     Lock();
 
-    nd_debug_printf("%s: entries: ids: %lu, flows: %lu\n",
+    nd_dprintf("%s: entries: ids: %lu, flows: %lu\n",
         tag.c_str(), ct_id_map.size(), ct_flow_map.size());
 
     Unlock();
