@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,9 +25,7 @@
 #include <stdexcept>
 #include <unordered_map>
 #include <vector>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
 
 #include <sys/types.h>
@@ -188,7 +186,7 @@ size_t ndDNSHintCache::purge(void)
     }
 
     if (purged > 0 && remaining > 0)
-        nd_debug_printf("Purged %u DNS cache entries, %u active.\n", purged, remaining);
+        nd_dprintf("Purged %u DNS cache entries, %u active.\n", purged, remaining);
 
     return purged;
 }
@@ -196,7 +194,7 @@ size_t ndDNSHintCache::purge(void)
 void ndDNSHintCache::load(void)
 {
     int rc;
-    time_t ttl;
+    long ttl;
     char header[1024], *host, *digest;
     size_t loaded = 0, line = 1;
     const char *filename = NULL;
@@ -239,7 +237,7 @@ void ndDNSHintCache::load(void)
             loaded++;
         }
 
-        nd_debug_printf("Loaded %u of %u DNS cache entries.\n",
+        nd_dprintf("Loaded %u of %u DNS cache entries.\n",
             map_ar.size(), loaded);
 
         pthread_mutex_unlock(&lock);
@@ -285,7 +283,7 @@ void ndDNSHintCache::save(void)
         pthread_mutex_unlock(&lock);
     }
 
-    nd_debug_printf("Saved %u of %u DNS cache entries.\n",
+    nd_dprintf("Saved %u of %u DNS cache entries.\n",
         saved, map_ar.size());
 
     fclose(hf);
