@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,9 +22,7 @@
 #include <vector>
 #include <map>
 #include <unordered_map>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
 
 #include <unistd.h>
@@ -38,6 +36,10 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 
+#define __FAVOR_BSD 1
+#include <netinet/tcp.h>
+#undef __FAVOR_BSD
+
 #include <pcap/pcap.h>
 
 #include <nlohmann/json.hpp>
@@ -48,23 +50,35 @@ using namespace std;
 #include "netifyd.h"
 
 #include "nd-ndpi.h"
+#ifdef _ND_USE_NETLINK
+#include "nd-netlink.h"
+#endif
 #include "nd-thread.h"
 #include "nd-json.h"
 #include "nd-util.h"
+#include "nd-flow.h"
 #include "nd-plugin.h"
 
 ndPlugin::ndPlugin(const string &tag)
     : ndThread(tag, -1), type(TYPE_BASE)
 {
-    nd_debug_printf("Plugin initialized: %s\n", tag.c_str());
+    nd_dprintf("Plugin initialized: %s\n", tag.c_str());
 }
 
 ndPlugin::~ndPlugin()
 {
-    nd_debug_printf("Plugin destroyed: %s\n", tag.c_str());
+    nd_dprintf("Plugin destroyed: %s\n", tag.c_str());
 }
 
-void ndPlugin::SetParams(const string uuid_dispatch, const ndJsonPluginParams &params)
+ndPluginSink::ndPluginSink(const string &tag)
+    : ndPlugin(tag) { }
+
+ndPluginSink::~ndPluginSink()
+{
+    nd_dprintf("Plugin sink destroyed: %s\n", tag.c_str());
+}
+
+void ndPluginSink::SetParams(const string uuid_dispatch, const ndJsonPluginParams &params)
 {
     Lock();
 
@@ -76,7 +90,7 @@ void ndPlugin::SetParams(const string uuid_dispatch, const ndJsonPluginParams &p
     Unlock();
 }
 
-bool ndPlugin::PopParams(string &uuid_dispatch, ndJsonPluginParams &params)
+bool ndPluginSink::PopParams(string &uuid_dispatch, ndJsonPluginParams &params)
 {
     bool popped = false;
 
@@ -96,7 +110,7 @@ bool ndPlugin::PopParams(string &uuid_dispatch, ndJsonPluginParams &params)
     return popped;
 }
 
-void ndPlugin::GetReplies(
+void ndPluginSink::GetReplies(
     ndPluginFiles &files, ndPluginFiles &data, ndPluginReplies &replies)
 {
     Lock();
@@ -120,44 +134,44 @@ void ndPlugin::GetReplies(
     Unlock();
 }
 
-void ndPlugin::PushFile(const string &tag, const string &filename)
+void ndPluginSink::PushFile(const string &tag, const string &filename)
 {
     files[tag] = filename;
 }
 
-void ndPlugin::PushData(const string &tag, const string &data)
+void ndPluginSink::PushData(const string &tag, const string &data)
 {
     this->data[tag] = data;
 }
 
-void ndPlugin::PushReply(
+void ndPluginSink::PushReply(
     const string &uuid_dispatch, const string &key, const string &value)
 {
     replies[uuid_dispatch][key] = value;
 }
 
 ndPluginService::ndPluginService(const string &tag)
-    : ndPlugin(tag)
+    : ndPluginSink(tag)
 {
-    type = TYPE_SERVICE;
-    nd_debug_printf("Plugin service initialized: %s\n", tag.c_str());
+    type = ndPlugin::TYPE_SINK_SERVICE;
+    nd_dprintf("Plugin service initialized: %s\n", tag.c_str());
 }
 
 ndPluginService::~ndPluginService()
 {
-    nd_debug_printf("Plugin service destroyed: %s\n", tag.c_str());
+    nd_dprintf("Plugin service destroyed: %s\n", tag.c_str());
 }
 
 ndPluginTask::ndPluginTask(const string &tag)
-    : ndPlugin(tag)
+    : ndPluginSink(tag)
 {
-    type = TYPE_TASK;
-    nd_debug_printf("Plugin task initialized: %s\n", tag.c_str());
+    type = ndPlugin::TYPE_SINK_TASK;
+    nd_dprintf("Plugin task initialized: %s\n", tag.c_str());
 }
 
 ndPluginTask::~ndPluginTask()
 {
-    nd_debug_printf("Plugin task destroyed: %s\n", tag.c_str());
+    nd_dprintf("Plugin task destroyed: %s\n", tag.c_str());
 }
 
 void ndPluginTask::SetParams(const string uuid_dispatch, const ndJsonPluginParams &params)
@@ -166,18 +180,42 @@ void ndPluginTask::SetParams(const string uuid_dispatch, const ndJsonPluginParam
     this->uuid_dispatch = uuid_dispatch;
     Unlock();
 
-    ndPlugin::SetParams(uuid_dispatch, params);
+    ndPluginSink::SetParams(uuid_dispatch, params);
 }
 
 bool ndPluginTask::PopParams(ndJsonPluginParams &params)
 {
     string uuid_dispatch;
-    return ndPlugin::PopParams(uuid_dispatch, params);
+    return ndPluginSink::PopParams(uuid_dispatch, params);
 }
 
 void ndPluginTask::PushReply(const string &key, const string &value)
 {
-    ndPlugin::PushReply(uuid_dispatch, key, value);
+    ndPluginSink::PushReply(uuid_dispatch, key, value);
+}
+
+ndPluginDetection::ndPluginDetection(const string &tag)
+    : ndPlugin(tag)
+{
+    type = ndPlugin::TYPE_DETECTION;
+    nd_dprintf("Plugin detection initialized: %s\n", tag.c_str());
+}
+
+ndPluginDetection::~ndPluginDetection()
+{
+    nd_dprintf("Plugin detection destroyed: %s\n", tag.c_str());
+}
+
+ndPluginStats::ndPluginStats(const string &tag)
+    : ndPlugin(tag)
+{
+    type = ndPlugin::TYPE_STATS;
+    nd_dprintf("Plugin detection initialized: %s\n", tag.c_str());
+}
+
+ndPluginStats::~ndPluginStats()
+{
+    nd_dprintf("Plugin detection destroyed: %s\n", tag.c_str());
 }
 
 ndPluginLoader::ndPluginLoader(const string &so_name, const string &tag)
@@ -205,12 +243,12 @@ ndPluginLoader::ndPluginLoader(const string &so_name, const string &tag)
         throw ndPluginException(tag, "ndPluginInit");
     }
 
-    nd_debug_printf("Plugin loaded: %s: %s\n", tag.c_str(), so_name.c_str());
+    nd_dprintf("Plugin loaded: %s: %s\n", tag.c_str(), so_name.c_str());
 }
 
 ndPluginLoader::~ndPluginLoader()
 {
-    nd_debug_printf("Plugin dereferenced: %s\n", so_name.c_str());
+    nd_dprintf("Plugin dereferenced: %s\n", so_name.c_str());
     if (so_handle != NULL) dlclose(so_handle);
 }
 
