@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,9 +30,7 @@
 #include <queue>
 #include <deque>
 #include <sstream>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
 
 #include <sys/types.h>
@@ -246,7 +244,7 @@ void *ndSinkThread::Entry(void)
 
     nd_dprintf("%s: thread started.\n", tag.c_str());
 
-    while (terminate == false) {
+    while (ShouldTerminate() == false) {
         Lock();
 
         if (uploads.size() == 0) {
@@ -285,7 +283,7 @@ void *ndSinkThread::Entry(void)
 
         Unlock();
 
-        if (terminate == false && pending.size() > 0) Upload();
+        if (ShouldTerminate() == false && pending.size() > 0) Upload();
     }
 
     return NULL;
@@ -302,7 +300,7 @@ void ndSinkThread::Terminate(void)
         throw ndSinkThreadException(strerror(rc));
     }
 
-    terminate = true;
+    ndThread::Terminate();
 
     Unlock();
 }
@@ -458,7 +456,7 @@ void ndSinkThread::Upload(void)
 
     if (post_errors == nd_config.sink_max_post_errors) {
         free(nd_config.url_sink);
-        nd_config.url_sink = strdup(ND_URL_SINK);
+        nd_config.url_sink = strdup(nd_config.url_sink_provision);
         nd_printf("%s: reverted to default sink URL: %s\n", tag.c_str(),
             nd_config.url_sink);
 
@@ -607,7 +605,7 @@ void ndSinkThread::Upload(void)
         pending_size -= pending.front().second.size();
         pending.pop_front();
 
-        if (pending.size() == 0 || terminate)
+        if (pending.size() == 0 || ShouldTerminate())
             flush_queue = false;
         else {
             Lock();
