@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,9 +30,7 @@
 #include <string>
 #include <unordered_map>
 #include <vector>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
 
 #include <arpa/inet.h>
@@ -829,7 +827,7 @@ void *ndSocketThread::Entry(void)
 
     nd_dprintf("%s: started\n", __PRETTY_FUNCTION__);
 
-    while (! terminate) {
+    while (! ShouldTerminate()) {
         int rc_read = -1, rc_write = -1;
         int max_read_fd = -1, max_write_fd = -1;
 
