@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,21 +22,25 @@
 #include <vector>
 #include <map>
 #include <unordered_map>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
 
 #include <unistd.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <pthread.h>
+#ifdef HAVE_PTHREAD_NP_H
+#include <pthread_np.h>
+#endif
 #include <string.h>
 #include <errno.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
+#ifdef HAVE_SYS_CPUSET_H
+#include <sys/cpuset.h>
+#endif
 
 #include <pcap/pcap.h>
 
@@ -95,26 +99,20 @@ ndThread::ndThread(const string &tag, long cpu, bool ipc)
         throw ndThreadSystemException(__PRETTY_FUNCTION__, "socketpair", errno);
 
     if (cpu == -1) return;
-#if defined(HAVE_PTHREAD_ATTR_SETAFFINITY_NP) && defined(CPU_ALLOC)
-    long cpus = sysconf(_SC_NPROCESSORS_ONLN);
+#if defined(HAVE_PTHREAD_ATTR_SETAFFINITY_NP)
+#ifdef HAVE_SYS_CPUSET_H
+    typedef cpuset_t cpu_set_t;
+#endif
+    cpu_set_t cpuset;
 
-    if (cpu >= cpus) cpu = 0;
-
-    cpu_set_t *cpuset = CPU_ALLOC(cpus);
-    if (cpuset == NULL) return;
-
-    size_t size = CPU_ALLOC_SIZE(cpus);
-
-    CPU_ZERO_S(size, cpuset);
-    CPU_SET_S(cpu, size, cpuset);
+    CPU_ZERO(&cpuset);
+    CPU_SET(cpu, &cpuset);
 
     rc = pthread_attr_setaffinity_np(
         &attr,
-        CPU_COUNT_S(size, cpuset),
-        cpuset
+        sizeof(cpuset),
+        &cpuset
     );
-
-    CPU_FREE(cpuset);
 #endif
 }
 
@@ -186,10 +184,10 @@ void ndThread::SendIPC(uint32_t id)
 {
     ssize_t bytes_wrote = 0;
 
-    bytes_wrote = send(fd_ipc[1], &id, sizeof(uint32_t), 0);
+    bytes_wrote = send(fd_ipc[IPC_PE_WRITE], &id, sizeof(uint32_t), 0);
 
     if (bytes_wrote != sizeof(uint32_t)) {
-        nd_debug_printf("%s: Failed to send IPC message: %s\n",
+        nd_dprintf("%s: Failed to send IPC message: %s\n",
             tag.c_str(), strerror(errno));
     }
 }
@@ -199,10 +197,10 @@ uint32_t ndThread::RecvIPC(void)
     uint32_t id = 0;
     ssize_t bytes_read = 0;
 
-    bytes_read = recv(fd_ipc[0], &id, sizeof(uint32_t), 0);
+    bytes_read = recv(fd_ipc[IPC_PE_READ], &id, sizeof(uint32_t), 0);
 
     if (bytes_read != sizeof(uint32_t)) {
-        nd_debug_printf("%s: Failed to receive IPC message: %s\n",
+        nd_dprintf("%s: Failed to receive IPC message: %s\n",
             tag.c_str(), strerror(errno));
     }
 
